<?php
/**
 * @file
 * Administration file for the Format Currencies project.
 */

/**
 * Settings form.
 */
function format_currency_settings($form, &$form_state) {
  $currencies = array();

  $query = db_select('currencies', 'c')
    ->fields('c', array('code', 'name'))
    ->orderBy('name')
    ->condition('c.status', FORMAT_CURRENCY_ACTIVE);
  $result = $query->execute();

  foreach ($result as $currency) {
    $currencies[$currency->code] = $currency->name;
  }

  if (!empty($currencies)) {
    $form['format_currency_test_wrapper'] = array(
      '#type' => 'fieldset',
      '#title' => t('Test output:'),
      '#prefix' => '<div id="format-currency-wrapper" class="container-inline">',
      '#suffix' => '</div>',
    );
    $form['format_currency_test_wrapper']['format_currency_test_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Display'),
      '#ajax' => array(
        'callback' => 'format_currency_ajax',
        'wrapper' => 'format-currency-test',
      ),
    );
    $form['format_currency_test_wrapper']['format_currency_test_select'] = array(
      '#type' => 'select',
      '#title' => t('Currency'),
      '#options' => $currencies,
    );
    $form['format_currency_test_wrapper']['format_currency_test'] = array(
      '#type' => 'textfield',
      '#description' => theme('format_currency', array('price' => 100, 'param' => 'GBP')),
      '#prefix' => '<div id="format-currency-test">',
      '#suffix' => '</div>',
      '#default_value' => isset($form_state['post']['format_currency_test']) ? $form_state['post']['format_currency_test'] : 100,
      '#size' => 10,
    );
    if (!empty($form_state['values']['format_currency_test'])) {
      $form['format_currency_test_wrapper']['format_currency_test']['#description'] = theme('format_currency', array('price' => $form_state['values']['format_currency_test'], 'param' => $form_state['values']['format_currency_test_select']));
    }
  }
  $form['format_currency_display_code'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display Currency Code'),
    '#description' => t('Whether to display the Currency Code (eg. "USD &#36;123.45") before a formatted price.'),
    '#default_value' => variable_get('format_currency_display_code'),
  );
  $form['format_currency_display_symbol'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use symbol rather than HTML entity'),
    '#description' => t('The database table holds both a symbol and HTML entity for Currencies, check the box to use the symbol over the entity.'),
    '#default_value' => variable_get('format_currency_display_symbol'),
  );
  $form['format_currency_round_whole_numbers'] = array(
    '#type' => 'checkbox',
    '#title' => t('Round whole numbers'),
    '#description' => t('Instead of showing &pound;100.00, checking this box will display the price as &pound;100. NOTE this only affects whole numbers &pound;100.01 will still display as normal.'),
    '#default_value' => variable_get('format_currency_round_whole_numbers'),
  );
  $form['format_currency_prefer_coinage'] = array(
    '#type' => 'checkbox',
    '#title' => t('Prefer coinage when displaying a price less than ONE unit'),
    '#description' => t('If displaying a price which is less than ONE unit, checking this box will use the coinage symbol if it is present in the configuration.
      <br />Eg. Rather than display &#36;0.99, display 99&cent;.'),
    '#default_value' => variable_get('format_currency_prefer_coinage'),
  );

  return system_settings_form($form);
}

/**
 * Overview of currencies currently in the database.
 */
function format_currency_admin($form, &$form_state) {
  drupal_add_css(drupal_get_path('module', 'format_currency') . '/format_currency.css');

  $query = db_select('currencies', 'c')->extend('PagerDefault')->extend('TableSort');

  $currencies = $query
    ->fields('c', drupal_schema_fields_sql('currencies'))
    ->limit(25)
    ->orderByHeader(_format_currency_admin_header())
    ->execute();

  $rows = array();

  foreach ($currencies as $currency) {
    $num = (int) $currency->num;

    $row = array(
      'name' => '<a href="#' . check_plain($currency->code) . '" name="' . check_plain($currency->code) . '" class="format-currency-tooltip">' . check_plain($currency->code) . ((boolean) $currency->status == FORMAT_CURRENCY_EXPIRED ? '*' : '') . '<span>' . check_plain($currency->name) . '</span></a>',
      'num' => !empty($num) ? str_pad($num, 3, 0, STR_PAD_LEFT) : '',
      'symbol' => check_plain($currency->symbol),
      'entity' => check_plain($currency->entity),
      'symbol_position' => _format_currency_symbol_position($currency->symbol_position),
      'coinage_symbol' => check_plain($currency->coinage_symbol),
      'coinage_entity' => check_plain($currency->coinage_entity),
      'coinage_position' => _format_currency_symbol_position($currency->coinage_position),
      'thousands_separator' => check_plain($currency->thousands_separator),
      'decimal_separator' => check_plain($currency->decimal_separator),
      'decimal_places' => check_plain($currency->decimal_places),
      'base' => check_plain($currency->base),
      l(t('edit'), "admin/config/regional/format_currency/$currency->code/edit", array('query' => drupal_get_destination())),
      l(t('delete'), "admin/config/regional/format_currency/$currency->code/delete", array('query' => drupal_get_destination())),
    );

    $rows[] = $row;
  }

  $form['currencies'] = array(
    '#rows' => $rows,
    '#header' => _format_currency_admin_header(),
    '#theme' => 'table',
    '#empty' => t('No currencies are currently in the database.'),
  );

  $form['new_link'] = array(
    '#type' => 'item',
    '#prefix' => '<div id="format-currency-admin-links">',
    '#markup' => l(t('Add new currency'), 'admin/config/regional/format_currency/add'),
    '#suffix' => '</div>',
  );

  $form['pager'] = array(
    '#type' => 'item',
    '#markup' => theme('pager', array('limit' => 25)),
  );

  return $form;
}

/**
 * Confirmation form to check if a user wants to rebuild their currencies table.
 */
function format_currency_repopulate($form, &$form_state) {
  return confirm_form(array(), t('Are you sure you want rebuild your currency table?'), 'admin/config/regional/format_currency/overview', t('<strong>Caution</strong>: This action is permanent and cannot be undone, it will erase all customisation you have made to your currency configuration.'));
}

/**
 * Submission callback for the format_currency_repopulate form.
 */
function format_currency_repopulate_submit($form, &$form_state) {
  $batch = array(
    'title' => t('Rebuilding currency table'),
    'operations' => array(
      array('_format_currency_batch_repopulate', array(25)),
    ),
  );

  // Start up the batch process.
  batch_set($batch);
  batch_process('admin/config/regional/format_currency/overview');
}

/**
 * Adds a new currency to the database.
 */
function format_currency_add($form, &$form_state) {
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency name'),
    '#maxlength' => 64,
  );
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency code'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['num'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency number'),
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol'),
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['entity'] = array(
    '#type' => 'textfield',
    '#title' => t('HTML Entity'),
    '#size' => 10,
    '#maxlength' => 32,
  );
  $form['symbol_position'] = array(
    '#type' => 'select',
    '#title' => t('Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => FORMAT_CURRENCY_LEFT,
  );
  $form['coinage_symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage Symbol'),
    '#size' => 15,
    '#maxlength' => 15,
  );
  $form['coinage_entity'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage HTML Entity'),
    '#size' => 10,
    '#maxlength' => 10,
  );
  $form['coinage_position'] = array(
    '#type' => 'select',
    '#title' => t('Coinage Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => FORMAT_CURRENCY_RIGHT,
  );
  $form['thousands_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Thousands Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => ',',
  );
  $form['decimal_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => '.',
  );
  $form['decimal_places'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Places'),
    '#required' => TRUE,
    '#size' => 3,
    '#maxlength' => 2,
    '#default_value' => 2,
  );
  $form['base'] = array(
    '#type' => 'textfield',
    '#title' => t('Base'),
    '#required' => TRUE,
    '#size' => 6,
    '#maxlength' => 6,
    '#default_value' => 100,
  );
  $form['status'] = array(
    '#type' => 'select',
    '#title' => t('Currency Status'),
    '#options' => array(
      FORMAT_CURRENCY_EXPIRED => t('Expired'),
      FORMAT_CURRENCY_ACTIVE => t('Active'),
    ),
    '#default_value' => FORMAT_CURRENCY_ACTIVE,
  );

  return confirm_form($form, t('Add New Currency'), 'admin/config/regional/format_currency/overview', ' ', t('Add'));
}

/**
 * Validation callback for the format_currency_add form.
 */
function format_currency_add_validate(&$form, &$form_state) {
  if ((bool) db_query_range('SELECT 1 FROM {currencies} WHERE code = :code', 0, 1, array(':code' => $form_state['values']['code']))->fetchField()) {
    form_set_error('code', t('Error this currency code already exists.'));
  }
  if (!empty($form_state['values']['num']) && !is_numeric($form_state['values']['num'])) {
    form_set_error('decimal_places', t('Currency number field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['decimal_places'])) {
    form_set_error('decimal_places', t('Decimal Place field can be numeric only.'));
  }
}

/**
 * Submission callback for the format_currency_add form.
 */
function format_currency_add_submit($form, &$form_state) {
  drupal_write_record('currencies', $form_state['values']);
  drupal_set_message(t('Successfully added currency %currency.', array('%currency' => $form_state['values']['code'])));
  $form_state['redirect'] = 'admin/config/regional/format_currency/overview';
}

/**
 * Form to edit a given currency in the database.
 */
function format_currency_edit($form, &$form_state, $currency) {
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency name'),
    '#maxlength' => 64,
    '#default_value' => $currency->name,
  );
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency code'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->code,
  );
  $form['num'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency number'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->num,
  );
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->symbol,
  );
  $form['entity'] = array(
    '#type' => 'textfield',
    '#title' => t('HTML Entity'),
    '#size' => 10,
    '#maxlength' => 32,
    '#default_value' => $currency->entity,
  );
  $form['symbol_position'] = array(
    '#type' => 'select',
    '#title' => t('Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => $currency->symbol_position,
  );
  $form['coinage_symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage Symbol'),
    '#size' => 15,
    '#maxlength' => 15,
    '#default_value' => $currency->coinage_symbol,
  );
  $form['coinage_entity'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage HTML Entity'),
    '#size' => 10,
    '#maxlength' => 10,
    '#default_value' => $currency->coinage_entity,
  );
  $form['coinage_position'] = array(
    '#type' => 'select',
    '#title' => t('Coinage Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => $currency->coinage_position,
  );
  $form['thousands_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Thousands Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->thousands_separator,
  );
  $form['decimal_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->decimal_separator,
  );
  $form['decimal_places'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Places'),
    '#required' => TRUE,
    '#size' => 3,
    '#maxlength' => 2,
    '#default_value' => $currency->decimal_places,
  );
  $form['base'] = array(
    '#type' => 'textfield',
    '#title' => t('Base'),
    '#required' => TRUE,
    '#size' => 6,
    '#maxlength' => 6,
    '#default_value' => $currency->base,
  );
  $form['status'] = array(
    '#type' => 'select',
    '#title' => t('Currency Status'),
    '#options' => array(
      FORMAT_CURRENCY_EXPIRED => t('Expired'),
      FORMAT_CURRENCY_ACTIVE => t('Active'),
    ),
    '#default_value' => $currency->status,
  );
  $form['old_code'] = array(
    '#type' => 'hidden',
    '#value' => $currency->code,
  );

  return confirm_form($form, t('Edit Currency %currency:', array('%currency' => $currency->code)), 'admin/config/regional/format_currency/overview', ' ', t('Edit'));
}

/**
 * Validation callback for the format_currency_edit form.
 */
function format_currency_edit_validate(&$form, &$form_state) {
  if ($form_state['values']['old_code'] != $form_state['values']['code'] && db_result(db_query("SELECT 1 FROM {currencies} WHERE code = '%s'", $form_state['values']['code']))) {
    form_set_error('code', t('Error this currency code already exists.'));
  }
  if (!empty($form_state['values']['num']) && !is_numeric($form_state['values']['num'])) {
    form_set_error('decimal_places', t('Currency number field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['base'])) {
    form_set_error('base', t('Base field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['decimal_places'])) {
    form_set_error('decimal_places', t('Decimal Place field can be numeric only.'));
  }
}

/**
 * Submission callback for the format_currency_edit form.
 */
function format_currency_edit_submit($form, &$form_state) {
  drupal_write_record('currencies', $form_state['values'], 'code');
  drupal_set_message(t('Successfully altered currency %currency.', array('%currency' => $form_state['values']['old_code'])));
  $form_state['redirect'] = 'admin/config/regional/format_currency/overview';
}

/**
 * Form to confirm deletion of a currency from the database.
 */
function format_currency_delete($form, &$form_state, $currency) {
  $form['code'] = array(
    '#type' => 'hidden',
    '#value' => $currency->code,
  );

  return confirm_form($form, t('Are you sure you want to delete currency %currency?', array('%currency' => $currency->code)), 'admin/config/regional/format_currency/overview');
}

/**
 * Submission callback for the format_currency_delete form.
 */
function format_currency_delete_submit($form, &$form_state) {
  db_delete('currencies')
    ->condition('code', $form_state['values']['code'])
    ->execute();
  drupal_set_message(t('Successfully deleted currency %currency.', array('%currency' => $form_state['values']['code'])));
  $form_state['redirect'] = 'admin/config/regional/format_currency/overview';
}

/**
 * AJAX callback to alter the format_currency_test description to show themed price output.
 */
function format_currency_ajax($form, $form_state) {
  return $form['format_currency_test_wrapper']['format_currency_test'];
}

/**
 * Helper form containing the admin overview table header.
 */
function _format_currency_admin_header() {
  return array(
    array('data' => t('Code'), 'field' => 'code', 'sort' => 'asc'),
    array('data' => t('Num'), 'field' => 'num'),
    t('Symbol'),
    t('Entity'),
    t('Symbol Position'),
    t('Coinage Symbol'),
    t('Coinage Entity'),
    t('Coinage Position'),
    t('Thousands Separator'),
    t('Decimal Separator'),
    t('Decimal Places'),
    t('Base'),
    array('data' => t('Operations'), 'colspan' => 2),
  );
}